@@ -1,23 +1,25 @@
 package CPAN;
 # This is the object that is used as global variable.
 # If you want to inherit from CPAN, just change the constructor
-use vars qw{$META $Signal $End};
+use vars qw{$META $Signal $End $Suppress_readline};
 
-$VERSION = '0.37a';
+$VERSION = '0.39a';
 
-# $Id: CPAN.pm,v 1.58 1996/09/23 14:03:23 k Exp k $
-my $version = substr q$Revision: 1.58 $, 10;
+# $Id: CPAN.pm,v 1.60 1996/09/27 12:48:56 k Exp $
+my $version = substr q$Revision: 1.60 $, 10;
 
 BEGIN {require 5.002;}
-use Term::ReadLine;
+
 use Carp ();
 
 # term
 
-my $suppress_readline = shift || 0;
+$Suppress_readline = shift || !-t STDIN; # undocumented how to turn off ReadLine
 my $term;
-if ($suppress_readline) {
+if ($Suppress_readline) {
 } else {
+    require Term::ReadLine;
+    import Term::ReadLine;
     $term = new Term::ReadLine 'CPAN Monitor';
     $readline::rl_completion_function = 
     $readline::rl_completion_function = 'CPAN::Complete::complete';
@@ -242,8 +244,9 @@ sub shell {
     no strict;
     $META->checklock();
     my $cwd = Cwd::cwd();
-    my $rl_avail = $suppress_readline ? "suppressed" :
-	defined %Term::ReadLine::Perl:: ? "enabled" :
+    # How should we determine if we have more than stub ReadLine enabled?
+    my $rl_avail = $Suppress_readline ? "suppressed" :
+	defined &Term::ReadLine::Perl::readline ? "enabled" :
 	    "available (get Term::ReadKey and Term::ReadLine::Perl)";
 
     print qq{
@@ -252,7 +255,7 @@ Readline support $rl_avail
 
 } unless $CPAN::Config->{'inhibit-startup-message'} ;
     while () {
-	if ($suppress_readline) {
+	if ($Suppress_readline) {
 	    print $prompt;
 	    last unless defined (chomp($_ = <>));
 	} else {
@@ -269,7 +272,6 @@ Readline support $rl_avail
 	} elsif (/^q$/i) {
 	    last;
 	} elsif (/./) {
-#	    $term->addhistory($_) if /\S/ && ! $suppress_readline;
 	    my @line = split;
 	    my $command = shift @line;
 	    eval { CPAN::Shell->$command(@line) };
@@ -312,7 +314,8 @@ u                             uninstalled modules
 make      modules,    make
 test      dists or    make test (implies make)
 install   bundles     make install (implies test)
-                      
+      
+reload    index|cpan  load most recent of indices or CPAN.pm                
 h                     display this menu
 o         various     set and query options
 !         perl-code   eval a perl command
@@ -364,6 +367,7 @@ sub o {
 	print "CPAN::Config options:\n";
 	while (($k,$v) = each %$CPAN::Config) {
 	    if (ref $v) {
+		printf "    %-18s\n", $k;
 		print map {"\t$_\n"} @{$v};
 	    } else {
 		printf "    %-18s %s\n", $k, $v;
@@ -1034,7 +1038,7 @@ sub get {
     my($self) = @_;
   EXCUSE: {
 	my @e;
-	defined $self->{'build_dir'} and push @e, "Stored in $self->{'build_dir'}";
+	exists $self->{'build_dir'} and push @e, "Unwrapped into directory $self->{'build_dir'}";
 	print join "", map {"  $_\n"} @e and return if @e;
     }
     my($local_file);
@@ -1076,7 +1080,7 @@ sub get {
 		$self->{unwrapped} = "NO";
 	    }
 	}
-	# Let's check if the author was nice to us.
+	# Let's check if the package has his own directory.
 	opendir DIR, "." or Carp::croak("Weird: couldn't opendir .: $!");
 	my @readdir = grep $_ !~ /^\.\.?$/, readdir DIR; ### MAC??
 	closedir DIR;
@@ -1084,6 +1088,7 @@ sub get {
 	if (@readdir == 1 && -d $readdir[0]) {
 	    $distdir = $readdir[0];
 	    $packagedir = $CPAN::META->catdir($builddir,$distdir);
+	    -d $packagedir and print "Removing previously used CPAN build directory\n";
 	    File::Path::rmtree($packagedir);
 	    rename($distdir,$packagedir) or Carp::confess("Couldn't rename $distdir to $packagedir");
 	} else {
@@ -1109,16 +1114,22 @@ sub get {
 	}
 	my($makefilepl) = $CPAN::META->catfile($packagedir,"Makefile.PL");
 	unless (-f $makefilepl) {
-	    my $fh = IO::File->new(">$makefilepl") or Carp::croak("Could not open >$makefilepl");
-	    my $cf = $self->called_for || "unknown";
-	    $fh->print(qq{
+	    my($configure) = $CPAN::META->catfile($packagedir,"Configure");
+	    if (-f $configure) {
+		# do we have anything to do?
+		$self->{'configure'} = $configure;
+	    } else {
+		my $fh = IO::File->new(">$makefilepl") or Carp::croak("Could not open >$makefilepl");
+		my $cf = $self->called_for || "unknown";
+		$fh->print(qq{
 # This Makefile.PL has been autogenerated by the module CPAN.pm
 # Autogenerated on: }.scalar localtime().qq{
-		use ExtUtils::MakeMaker;
-		WriteMakefile(NAME => q[$cf]);
+		    use ExtUtils::MakeMaker;
+		    WriteMakefile(NAME => q[$cf]);
 });
-	    print qq{Package comes without Makefile.PL.\n}.
-		qq{  Writing one on our own (calling it $cf)\n};
+		print qq{Package comes without Makefile.PL.\n}.
+		    qq{  Writing one on our own (calling it $cf)\n};
+	    }
 	}
     } else {
 	$self->{archived} = "NO";
@@ -1222,8 +1233,8 @@ sub make {
 	  my @e;
 	  $self->{archived} eq "NO" and push @e, "Is neither a tar nor a zip archive.";
 	  $self->{unwrapped} eq "NO"   and push @e, "had problems unarchiving. Please build manually";
-	  $self->{writemakefile} && $self->{writemakefile} eq "NO" and push @e, "Had some problem writing Makefile";
-	  defined $self->{'make'}     and push @e, "Has already been processed within this session";
+	  exists $self->{writemakefile} && $self->{writemakefile} eq "NO" and push @e, "Had some problem writing Makefile";
+	  defined $self->{'make'} and push @e, "Has already been processed within this session";
 	  print join "", map {"  $_\n"} @e and return if @e;
      }
     print "\n  CPAN: Going to build ".$self->id."\n\n";
@@ -1231,11 +1242,17 @@ sub make {
     chdir $builddir or Carp::croak("Couldn't chdir $builddir: $!");
     $self->debug("Changed directory to $builddir") if $CPAN::DEBUG;
 
-    my $system = "$^X Makefile.PL $CPAN::Config->{makepl_arg}";
+    my $system;
+    if ($self->{'configure'}) {
+	$system = $self->{'configure'};
+    } else {
+	$system = "$^X Makefile.PL $CPAN::Config->{makepl_arg}";
+    }
     if (system($system)!=0) {
 	 $self->{writemakefile} = "NO";
 	 return;
     }
+    $self->{writemakefile} = "YES";
     return if $CPAN::Signal;
     $system = join " ", $CPAN::Config->{'make'}, $CPAN::Config->{make_arg};
     if (system($system)==0) {
@@ -1255,7 +1272,8 @@ sub test {
     print "Running make test\n";
     EXCUSE: {
 	  my @e;
-	  exists $self->{'make'} && $self->{'make'} eq 'NO' and push @e, "Make returned bad status";
+	  exists $self->{'make'} or push @e, "Make had some problems, maybe interrupted? Won't test";
+	  exists $self->{'make'} and $self->{'make'} eq 'NO' and push @e, "Oops, make had returned bad status";
 	  exists $self->{'build_dir'} or push @e, "Has no own directory";
 	  print join "", map {"  $_\n"} @e and return if @e;
      }
@@ -1278,8 +1296,9 @@ sub install {
     print "Running make install\n";
     EXCUSE: {
 	  my @e;
-	  exists $self->{'install'} and push @e, "Already done";
-	  exists $self->{'make'} && $self->{'make'} eq 'NO' and push @e, "Make returned bad status";
+	  exists $self->{'make'} or push @e, "Make had some problems, maybe interrupted? Won't install";
+	  exists $self->{'make'} and $self->{'make'} eq 'NO' and push @e, "Oops, make had returned bad status";
+	  exists $self->{'install'} and push @e, $self->{'install'} eq "YES" ? "Already done" : "Already tried without success";
 	  exists $self->{'build_dir'} or push @e, "Has no own directory";
 	  print join "", map {"  $_\n"} @e and return if @e;
      }
